#include "controle.h"

void controle::PROX_ESTADO(){

  if ( reset ) {
    atual.write(IDLE);
  }
  else {
	atual.write(prox.read());
  }

}

void controle::update_signals() {

	if(reset){
	    halt = false;
	    ce_int = false;
		rw_int = READ;
	}
	else{
		switch(atual.read()){
			case IDLE:
				{
				if(wait_r8){
					ce_int = false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
				}
				else{
					ce_int =false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
					wula = false;
					wnz = false;
					wcv = false;
					rw_int = READ;
				}
				break;
				}
			case FETCH:
				{
				if ( wait_r8 ) { // if processor in wait, keep in state and write nowhere
					ce_int =false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
					wula = false;
					wnz = false;
					wcv = false;
					rw_int = READ;
				}
				else { // if in FETCH just do fetch
					wreg = false;
					wsp = false;
					wpc = true;
					wir = true;
					mad = 1;
					mpc=2;
					ce_int =true;
					rw_int = READ;
					ms2 = false;
				}
				break;
				}
			case REG:
				{
				if ( wait_r8 ) { // if processor in wait, keep in state and write nowhere
					ce_int =false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
					wula = false;
					wnz = false;
					wcv = false;
				}
				else {
					wpc = false;
					wir = false;
					ce_int =false;
					rw_int = READ;
					wab = true;
					switch(ir.read().range(15,12)) {
						case ADD:
						case SUB:
						case AND:
						case OR:
						case XOR:
						case LD:
						case ST:
							ms2 = false;
							break;
						default:
							ms2 = true;
						break;
					}
				}

				break;
				}
			case ULA:
				{
				if ( wait_r8 ) { // if processor in wait, keep in state and write nowhere
					ce_int =false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
					wula = false;
					wnz = false;
					wcv = false;
				}
				else {
					wpc = false;
					wab = false;
					wula = true;
					switch( ir.read().range(15,12) ) {
						case ADD:
							ula = add;
							ma = false;
							mb = 0;
							wnz = true;
							wcv = true;
							break;
						case SUB:
							ula = sub;
							ma = false;
							mb = 0;
							wnz = true;
							wcv = true;
							break;
						case AND:
							ula = e;
							ma = false;
							mb = 0;
							wnz = true;
							break;
						case OR:
							ula = ou;
							ma = false;
							mb = 0;
							wnz = true;
							break;
			            case XOR:
				             ula = oux;
				             ma = false;
				             mb = 0;
				             wnz = true;
							break;
						case ADDI:
							ula = add;
							ma = true;
							mb = 0;
							wnz = true;
							wcv = true;
							break;
						case SUBI:
							ula = subi;
							ma = true;
							mb = 0;
							wnz = true;
							wcv = true;
							break;
						case LDL:
							ula = ldl;
							ma = true;
							mb = 0;
							break;
						case LDH:
							ula = ldh;
							ma = true;
							mb = 0;
							break;
						case LD:
							ula = add;
							ma = false;
							mb = 0;
							break;
						case ST:
							ula = add;
							ma = false;
							mb = 0;
							mad = 0;  // para o endereco estar estavel na subida do 4 ciclo
							ms2 = true;
							break;

						case SALTOD:
						case JMPD:
						case JSRD:
							ula = add;
							ma = true;
							mb = 1;
							mad = 2;  // JSRD, para o endereco estar estavel na subida do 4 ciclo
							break;

						case SALTOR:
							switch(ir.read().range(3,0)) {  // Seleciona o SALTOR
								case JMP:
								case JMPN:
								case JMPZ:
								case JMPC:
								case JMPV:
									ula = passa_opA;
									ma = false;
									break;
				                case JSRR:
									ula = add;
									mb = 1;   // PC vai para a pilha
									ma = false;
									mad = 2;  // para o endereco estar estavel na subida do 4 ciclo
									break;
				                case JSR:
									// rw=WRITE;
									// ce=false;
									// ula=passa_opB;
									// mb=1;
									// ma=false;
					                // mad = 2;  // para o endereco estar estavel na subida do 4 ciclo
									// msp=true;
									ula = passa_opA;
									mb = 1;   // PC vai para a pilha
									mad = 2;  // para o endereco estar estavel na subida do 4 ciclo

					                break;
								default:    // JMPR, JMPNR, JMPZR, JMPCR, JMPVR
									ula = add;
									ma = false;
									mb = 1;
									break;
							}
							break;

						case OUTRAS:
							switch(ir.read().range(3,0)) {
								case SL0:
									ula = sl0;
									ma = false;
									wnz = true;
									break;

								case SL1:
									ula = sl1;
									ma = false;
									wnz = true;
									break;

								case SR0:
									ula = sr0;
									ma = false;
									wnz = true;
									break;

								case SR1:
									ula = sr1;
									ma = false;
									wnz = true;
									break;

								case NOT:
									ula = not_opA;
									ma = false;
									wnz = true;
									break;

								case LDSP:
									ula = passa_opA;
									ma = false;
									break;

								case RTS:
									wpc=false;
									mpc=0;
									ula = inc_opB;
									mb = 2;
									mad = 0;
									rw_int =READ;
									break;

								case POP:
									ula = inc_opB;
									mb = 2;
									break;

				                case PUSH:
									ula = passa_opB;
									mb = 2;
									mad = 0;  // para o endereco estar estavel na subida do 4 ciclo
									break;

				                default:
									ula = add;
									wnz = false;
									wcv = false;
									break;
							}
							break;

						default:
							ula = add;
				            wnz = false;
				            wcv = false;
				            break;
					}
				}
				break;
				}
			case WBACK:
				{
				if ( wait_r8 ) { // if processor in wait, keep in state and write nowhere
					ce_int =false;
					wreg = false;
					wsp = false;
					wpc = false;
					wir = false;
					wula = false;
					wnz = false;
					wcv = false;
				}
				else {
					wula = false;
					wnz = false;
					wcv = false;
					switch( ir.read().range(15,12) ) {
						case ADD:
						case SUB:
						case AND:
						case OR:
						case XOR:
						case ADDI:
						case SUBI:
						case LDL:
						case LDH:
							wreg = true;
							mreg = false;
							ce_int =false;
							break;

						case LD:
							wreg = true;
							mreg = true;
							mad = 0;
							ce_int =true;
							rw_int = READ;
							break;

						case ST:
							wreg = false;
							rw_int = WRITE;
							ce_int =true;
							break;

						case JMPD:  // ok
							wpc = true;
							mpc = 1;
							break;

						case JSRD:
							wpc = true;
							mpc = 1;
							wsp = true;
							msp = true;
							ce_int =true;
							rw_int = WRITE;
							break;

			            case SALTOR:
							mpc = 1;
							mad = 0;
							switch( ir.read().range(3,0) ) {
				                case JMPR://ok
				                case JMP: //ok
									wpc = true;
									break;
				                case JMPNR://ok
				                case JMPN:// ok
									if ( flag.read()[N] == 1 )
										wpc = true;
									else
										wpc = false;
									break;

				                case JMPZR://ok
				                case JMPZ:// ok
					                if ( flag.read()[Z] == 1 )
										wpc = true;
									else
										wpc = false;
					                break;
								case JMPCR:
								case JMPC:
									if ( flag.read()[C] == 1 )
										wpc = true;
									else
										wpc = false;
									break;

								case JMPVR:
								case JMPV:
									if ( flag.read()[V] == 1 )
										wpc = true;
									else
										wpc = false;
									break;
								case JSRR: // ok
								case JSR: // ok
									// rw=WRITE;
									// ce=true;

									mad = 2;
									// wpc = true;
									// mpc = 1;
									// ula=passa_opA;
									// wsp = true;
									// msp = true;
									wpc = true;
									mpc = 1;
									wsp = true;
									msp = true;
									ce_int = true;
									rw_int = WRITE;
									break;
				                default:
					                wpc = false;
					                ce_int =false;
					                break;
				            }
							break;

			            case SALTOD:
			            mpc = 1;
			            switch( ir.read().range(11,10) ) {
			                case JMPND: // ok
				                if ( flag.read()[N] == 1 )
									wpc = true;
				                else
									wpc = false;
				                break;

							case JMPZD: // ok
								if ( flag.read()[Z] == 1 )
									wpc = true;
								else
									wpc = false;
								break;

							case JMPCD:
								if ( flag.read()[C] == 1 )
									wpc = true;
								else
									wpc = false;
								break;

							case JMPVD:
								if ( flag.read()[V] == 1 )
									wpc = true;
								else
									wpc = false;
								break;

							default:
								wpc = false;
								break;
						}
			            break;

			            case OUTRAS:
							switch( ir.read().range(3,0) ) {
				                case SL0:
				                case SL1:
				                case SR0:
				                case SR1:
				                case NOT:
					                wreg = true;
					                mreg = false;
					                ce_int =false;
									break;

								case LDSP:
									wsp = true;
									msp = false;
									wreg = false;
									ce_int =false;
									break;

								case POP:
									wreg = true;
									mreg = true;
									wsp = true;
									msp = false;
									mad = 0;
									ce_int =true;
									rw_int = READ;
									break;

								case PUSH:
									wreg = false;
									wsp = true;
									msp = true;
									ce_int =true;
									rw_int = WRITE;
									break;

								case RTS:
									wpc = true;
									mpc = 0;
									mad = 0;
									wsp = true;
									msp = false;
									ce_int =true;
									rw_int = READ;
									break;

								default:  // NOP
									wreg = false;
									ce_int =false;
									break;
							}
							break;

			            default:
							wreg = false;
							ce_int =false;
			            break;
			        }
				}
				break;
				}
			case FIM:  // if processor in halt, keep in halt and write nowhere
				{
				ce_int =false;
				wreg = false;
				wsp = false;
				wpc = false;
				wir = false;
				wula = false;
				wnz = false;
				wcv = false;
				rw_int = WRITE;

				halt = true;
				break;
				}
		}
	}
}

void controle::FSM() {

	if ( reset ) {
		prox.write(IDLE);
	}
	else {
		switch(atual.read()) {

			case IDLE:
				if ( wait_r8 ) prox.write(IDLE);
				else prox.write(FETCH);
				break;

			case FETCH:                     // 1
				if ( wait_r8 ) prox.write(FETCH);
				else prox.write(REG);
				break;

			case REG:                     // 2
				if ( wait_r8 ) prox.write(REG);
				else {
					if ( ir.read().range(15,12) == OUTRAS && ir.read().range(3,0) == HALT ) {
						prox.write(FIM);
					}
					else
						prox.write(ULA);
				}
				break;

			case ULA:
				if ( wait_r8 ) prox.write(ULA);
				else prox.write(WBACK);
				break;

		  case WBACK:
				if ( wait_r8 ) prox.write(WBACK);
				else prox.write(FETCH);
				break;

		  case FIM:  // if processor in halt, keep in halt and write nowhere
			prox.write(FIM);
			break;
		}
	}
}

void controle::update_ce_on_falling_edge() {
	if (reset)
		ce = false;
	else
		ce = ce_int & clock;
}

void controle::update_rw_on_falling_edge() {
	if (reset)
		rw = READ;
	else
		if(clock)
			rw = rw_int;
		else
			rw = READ;
}
