#ifndef _controle_h
#define _controle_h

#include "systemc.h"
#include "datapath.h"

SC_MODULE(controle) {

  sc_in< bool > clock, reset, wait_r8;
  sc_in< sc_uint<16> > ir;
  sc_in< sc_bv<4> > flag;


  sc_out< bool > ce, rw, halt,
    msp, mreg, ms2, ma, wpc, wsp, wir, wab, wula, wreg, wnz, wcv; //
  sc_out< sc_uint<2> > mpc, mad, mb;                  // microinstrucao
  sc_out< operacao > ula;                       //

  sc_bv<4> s_flag;
  sc_signal<sc_uint<4> > atual, prox; // estado atual da FSM
  sc_signal<bool> ce_int, rw_int;

  void PROX_ESTADO();
  void FSM();
  void update_ce_on_falling_edge();
  void update_rw_on_falling_edge();
  void update_signals();

  SC_CTOR(controle) {
    SC_METHOD(PROX_ESTADO);
    sensitive << reset.pos() << clock.pos();
	
    SC_METHOD(FSM);
    sensitive << reset.pos() << clock.neg() << wait_r8;
	
    SC_METHOD(update_signals);
    sensitive << reset << atual << wait_r8 << ir << flag;

    SC_METHOD(update_ce_on_falling_edge);
    sensitive << reset << clock << ce_int;

    SC_METHOD(update_rw_on_falling_edge);
    sensitive << reset << clock << rw_int;

  }
};

#endif
