#include "datapath.h"

//  flags de estado
void datapath::flags() {

  s_outula = outula.read();

  if ( reset )
    s_flag = 0; // zera todos os 4 flags

  else {

    if ( wnz ) {
      s_flag[N] = (s_outula[15]==1)?1:0;

      if ( s_outula & 0xFFFF )
        s_flag[Z] = false;
      else
        s_flag[Z] = true;
    }

    if ( wcv ) {

      s_flag[C] = fcarry(s_opA, s_opB,(s_opA[15] ^ s_opB[15])); // tavez seja or ao inves de xor

      if ( s_opA[15] == s_opB[15] )
        s_flag[V] = ((s_opA[15]==1)?1:0) ^ ((s_outula[15]==1)?1:0);
      else
            s_flag[V] = false;
    }
  }

  flag = s_flag;
}

void datapath::muxPC() {

  switch(mpc.read()) {
    case 0:
      dtpc = datain;
    break;

    case 1:
      dtpc = rula;
    break;

    case 2:
      //dtpc = pc.read() + 1;
      dtpc = full_adder(pc.read(),1,0);
    break;
  }
}

void datapath::muxSP() {

  sc_uint<16> um = 1;

  if ( msp )
    //dtsp = sp.read() - 1;
    dtsp = full_adder(sp.read(),~um,1);
  else
    dtsp = rula;
}

void datapath::mux_end() {

  switch(mad.read()) {
    case 0:
      endereco = rula;
    break;

    case 1:
      endereco = pc;
    break;

    case 2:
      endereco = sp;
    break;
  }
}

void datapath::muxOPA() {

  sc_uint<12> desloc;
  sc_uint<6> ext1 = 0x3F;
  sc_uint<4> ext2 = 0xF;
  sc_uint<8> zero = 0;

  if ( ma ) {
    desloc = ir.read().range(11,0);
    switch(ir.read().range(15,12)) {

      case JMPD:
      case SALTOD:
        if ( desloc[9] == 1 )
          opA = (ext1,desloc.range(9,0));
        else
          opA = (zero.range(5,0),desloc.range(9,0));
      break;

      case JSRD:
        if ( desloc[11] == 1 ) {
          opA = (ext2,desloc);
        }
        else
          opA = (zero.range(3,0),desloc);
      break;

	  case SALTOR:
	  	opA = rA;
	  break;

      default:
        opA = (zero.range(7,0),ir.read().range(7,0));
      break;
    }
  }
  else
    opA = rA;
}

void datapath::muxOPB() {

  switch(mb.read()) {
    case 0:
      opB = rB;
    break;

    case 1:
      opB = pc;
    break;

    case 2:
      opB = sp;
    break;
  }
}

void datapath::mux_dtreg() {

  if ( mreg )
    dtreg = datain;
  else
    dtreg = rula;
}

void datapath::mux_dataout() {

  if ( ir.read().range(15,12) == 0x0A ||  // ST
    (ir.read().range(15,12) == 0x0B && ir.read().range(3,0) == 0x0A) ) // PUSH
    dataout = s2;
  else
    dataout = opB;
}

void datapath::out_ula() {

  sc_uint<16> A, B;
  sc_uint<1> um, zero;

  A = opA.read();
  B = opB.read();

  switch(ula) {

    case add:
      //outula = A + B;
      outula = full_adder(A,B,0);
      s_opA = A;
      s_opB = B;
    break;

    case sub:
      //outula = A - B;
      outula = full_adder(A,~B,1);
      s_opA = A;
      s_opB = ~B;
    break;

    case subi:
      //outula = B - A;
      outula = full_adder(B,~A,1);
      s_opA = B;
      s_opB = ~A;
    break;

    case e:
      outula = A & B;
    break;

    case ou:
      outula = A | B;
    break;

    case oux:
      outula = A ^ B;
    break;

    case ldl:
      outula = (B.range(15,8), A.range(7,0));
    break;

    case ldh:
      outula = (A.range(7,0), B.range(7,0));
    break;

    case sl0:
            outula = (A.range(14,0),zero);
    break;

    case sl1:
      outula = (A.range(14,0),um);
    break;

    case sr0:
      outula = (zero,A.range(15,1));
    break;

    case sr1:
      outula = (um,A.range(15,1));
    break;

    case not_opA:
      outula = ~A;
    break;

    case passa_opA:
      outula = A;
    break;

    case passa_opB:
      outula = B;
    break;

    case inc_opB:
      //outula = B + 1;
      outula = full_adder(B,1,0);
    break;

    default:
      //outula = A + B;
      outula = full_adder(A,B,0);
    break;
  }
}

void datapath::out_ir() {

  instrucao = ir;

}

sc_uint<16> datapath::full_adder(sc_uint<16> a, sc_uint<16> b, sc_uint<1> cin) {

  sc_uint<1> carry, carry_ant;
  sc_uint<16> s;

  for( int w=0; w<16; w++ ) {
    if ( w == 0 )
      carry = cin;
    s[w] = a[w] ^ b[w] ^ carry;
    carry_ant = carry;
    carry = (a[w] & b[w]) | (a[w] & carry)| (b[w] & carry);
  }

  return s;
}

// Verifica se deu carry
bool datapath::fcarry(sc_uint<16> a, sc_uint<16> b, sc_uint<1> cin) {

  sc_uint<1> carry, carry_ant;
  sc_uint<16> s;

  for( int w=0; w<16; w++ ) {
    if ( w == 0 )
      carry = cin;
    carry = (a[w] & b[w]) | (a[w] & carry)| (b[w] & carry);
  }

  return (bool)carry;
}
