#ifndef _datapath_h
#define _datapath_h

// Gera vhdl
// Nao da pra usar + nem - para sintetizar pra virtex

#include "systemc.h"
#include "bc_regs.h"
#include "pack.h"

SC_MODULE(datapath) {

  sc_in< bool > clock, reset, msp, mreg, ms2, ma, wpc, wsp, wir, wab, wula, wreg, wnz, wcv; //
  sc_in< sc_uint<2> > mpc, mad, mb;                 // microinstrucao
  sc_in< operacao > ula;                        //
  sc_in< sc_uint<16> > datain;


  sc_out< sc_uint<16> > endereco, dataout, instrucao;
  sc_out< sc_bv<4> > flag;


  sc_signal< sc_uint<16> > dtreg, dtpc, dtsp, s1, s2, outula, pc, sp, ir, rA, rB, rula, opA, opB, somaA, somaB, soma;


  sc_uint<16> s_opA, s_opB, s_outula; // variaveis
  sc_bv<4> s_flag;          // auxiliares

  bc_regs *br;
  reg16 *rpc, *rsp, *rir, *reg_a, *reg_b, *reg_ula;

  void flags();
  void muxPC();
  void muxSP();
  void mux_end();
  void muxOPA();
  void muxOPB();
  void mux_dtreg();
  void mux_dataout();
  void out_ula();
  void out_ir();

  sc_uint<16> full_adder(sc_uint<16> a, sc_uint<16> b, sc_uint<1> cin);
  bool fcarry(sc_uint<16> a, sc_uint<16> b, sc_uint<1> cin);


  SC_CTOR(datapath) {
    SC_METHOD(flags);
    sensitive << reset.pos();
    sensitive << clock.neg();

    SC_METHOD(muxPC);
    sensitive << datain << rula << pc << mpc;

    SC_METHOD(muxSP);
    sensitive << rula << sp << msp;

    SC_METHOD(mux_end);
    sensitive << rula << pc << sp << mad;

    SC_METHOD(muxOPA);
    sensitive << ir << rA << ma;

    SC_METHOD(muxOPB);
    sensitive << rB << pc << sp << mb;

    SC_METHOD(mux_dtreg);
    sensitive << rula << datain << mreg;

    SC_METHOD(mux_dataout);
    sensitive << opB << s2 << ir;

    SC_METHOD(out_ula);
    sensitive << opA << opB << ula;

    SC_METHOD(out_ir);
    sensitive << ir;

    // registradores do bloco de dados: banco, pc, sp, ir, rA, rB, rula

    br = new bc_regs("banco");
    rpc = new reg16("PC");
    rsp = new reg16("SP");
    rir = new reg16("IR");
    reg_a = new reg16("A");
    reg_b = new reg16("B");
    reg_ula = new reg16("RULA");

    br->clock(clock);
    br->reset(reset);
    br->wreg(wreg);
    br->ms2(ms2);
    br->ir(ir);
    br->inREG(dtreg);
    br->source1(s1);
    br->source2(s2);

    rpc->clock(clock);
    rpc->reset(reset);
    rpc->ce(wpc);
    rpc->D(dtpc);
    rpc->Q(pc);

    rsp->clock(clock);
    rsp->reset(reset);
    rsp->ce(wsp);
    rsp->D(dtsp);
    rsp->Q(sp);

    rir->clock(clock);
    rir->reset(reset);
    rir->ce(wir);
    rir->D(datain);
    rir->Q(ir);

    reg_a->clock(clock);
    reg_a->reset(reset);
    reg_a->ce(wab);
    reg_a->D(s1);
    reg_a->Q(rA);

    reg_b->clock(clock);
    reg_b->reset(reset);
    reg_b->ce(wab);
    reg_b->D(s2);
    reg_b->Q(rB);

    reg_ula->clock(clock);
    reg_ula->reset(reset);
    reg_ula->ce(wula);
    reg_ula->D(outula);
    reg_ula->Q(rula);
  }

};


#endif
