// Test Bench
#include "processador.h"
#include "sinais.h"

int sc_main(int argc, char* argv[]) {

	sc_time my_period (20, SC_NS);
	sc_clock clk("clock", my_period, 0.5, SC_ZERO_TIME, false);

	sc_signal< sc_uint<16> > datain, dataout, endereco;
	sc_signal < bool > reset, rw, ce, halt, wait_r8;

	sinais s1("sinais");
	processador p("r8");


	p.clock(clk);
	s1.clock(clk);

	p.reset(reset);
	s1.reset(reset);

	p.datain(datain);
	s1.dataout(datain);

	p.dataout(dataout);
	s1.datain(dataout);

	p.endereco(endereco);
	s1.addr(endereco);

	p.ce(ce);
	s1.ce(ce);

	p.rw(rw);
	s1.rw(rw);

	p.wait_r8(wait_r8);
	s1.wait_r8(wait_r8);

	p.halt(halt);

  sc_trace_file *tf = sc_create_vcd_trace_file("trace_file");
	sc_trace(tf, reset, "RESET");
	sc_trace(tf, clk.signal(), "CLOCK");
	sc_trace(tf, endereco, "ENDERECO");
	sc_trace(tf, dataout, "DATAOUT");
	sc_trace(tf, p.dp->datain, "DATAIN");
	sc_trace(tf, p.dp->pc, "PC");
	sc_trace(tf, p.dp->sp, "SP");
	sc_trace(tf, p.dp->ir, "IR");
	sc_trace(tf, p.c->atual, "EATUAL");
	sc_trace(tf, p.dp->rula, "RULA");
	sc_trace(tf, p.dp->br->reg[0]->Q, "REG(0)");
	sc_trace(tf, p.dp->br->reg[1]->Q, "REG(1)");
	sc_trace(tf, p.dp->br->reg[2]->Q, "REG(2)");
	sc_trace(tf, p.dp->br->reg[3]->Q, "REG(3)");
	sc_trace(tf, p.dp->br->reg[4]->Q, "REG(4)");
	sc_trace(tf, p.dp->br->reg[5]->Q, "REG(5)");
	sc_trace(tf, p.dp->br->reg[6]->Q, "REG(6)");
	sc_trace(tf, s1.ram[0], "RAM(0)");
	sc_trace(tf, s1.ram[1], "RAM(1)");
	sc_trace(tf, s1.ram[2], "RAM(2)");
	sc_trace(tf, s1.ram[3], "RAM(3)");
	sc_trace(tf, s1.ram[4], "RAM(4)");
	sc_trace(tf, s1.ram[5], "RAM(5)");
	sc_trace(tf, s1.ram[6], "RAM(6)");
	sc_trace(tf, s1.ram[7], "RAM(7)");
	sc_trace(tf, s1.ram[8], "RAM(8)");
	sc_trace(tf, s1.ram[9], "RAM(9)");
	sc_trace(tf, s1.ram[10], "RAM(10)");
	sc_trace(tf, s1.ram[11], "RAM(11)");
	sc_trace(tf, s1.ram[12], "RAM(12)");
	sc_trace(tf, s1.ram[13], "RAM(13)");
	sc_trace(tf, s1.ram[14], "RAM(14)");
	sc_trace(tf, s1.ram[15], "RAM(15)");
	sc_trace(tf, s1.ram[16], "RAM(16)");
	sc_trace(tf, rw, "RW");
	sc_trace(tf, ce, "CE");
	sc_trace(tf, halt,"HALT");

	sc_start(10000, SC_NS);
	sc_close_vcd_trace_file(tf);

	return 0;
}
